/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.config.type;

import java.lang.reflect.*;
import java.util.*;
import java.util.logging.*;

import com.caucho.config.*;
import com.caucho.config.attribute.*;
import com.caucho.util.*;
import com.caucho.xml.QName;
import com.caucho.webbeans.component.*;
import com.caucho.webbeans.manager.*;

/**
 * Represents an array of values for configuration.
 */
public class ArrayType extends ConfigType
{
  private static final L10N L = new L10N(ArrayType.class);
  private static final Logger log
    = Logger.getLogger(ListType.class.getName());

  private final ConfigType _componentType;
  private final Class _componentClass;
  private final Class _type;

  public ArrayType(ConfigType componentType, Class componentClass)
  {
    _componentType = componentType;
    _componentClass = componentClass;

    Class type = null;
    try {
      type = Array.newInstance(componentClass, 0).getClass();
    } catch (Exception e) {
    }

    _type = type;
  }

  /**
   * Returns the given type.
   */
  public Class getType()
  {
    return _type;
  }

  /**
   * Creates a new instance
   */
  @Override
  public Object create(Object parent)
  {
    return new ArrayList();
  }

  /**
   * Returns the attribute based on the given name.
   */
  @Override
  public Attribute getAttribute(QName name)
  {
    // XXX: type
    
    return TypeFactory.getFactory().getListAttribute(name);
  }

  /**
   * Replaces the object.
   */
  @Override
  public Object replaceObject(Object value)
  {
    if (value == null)
      return null;

    List list = (List) value;

    Object []array
      = (Object []) Array.newInstance(_componentClass, list.size());

    list.toArray(array);
    
    return array;
  }
  
  /**
   * Converts the string to the given value.
   */
  public Object valueOf(String text)
  {
    throw new ConfigException(L.l("Can't convert to '{0}' from '{1}'.",
				  _type.getName(), text));
  }

  public String toString()
  {
    return getClass().getSimpleName() + "[" + _componentClass.getName() + "]";
  }
}
